﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.IO;
using System.Reflection;

namespace MyNamespace
{
    class MyApplication
    {
        [ImportMany(typeof(MyInterface))]
        private IEnumerable<MyInterface> Extensions { get; set; }

        public MyApplication()
        {
            var catalog = new DirectoryCatalog(Path.GetDirectoryName(Assembly.GetEntryAssembly().Location));
            var container = new CompositionContainer(catalog);
            container.SatisfyImportsOnce(this);
        }

        private void Run()
        {
            Console.WriteLine("Application: Version={0}", Assembly.GetEntryAssembly().GetName().Version.ToString());
            foreach (var extension in Extensions)
            {
                Console.WriteLine("Extension: Name={0} Version={1}", extension.Name, extension.GetVersion());
            }
        }

        static void Main(string[] args)
        {
            try
            {
                var application = new MyApplication();
                application.Run();
            }
            catch (Exception ex)
            {
                Console.WriteLine("Exception: {0}", ex);
                if (ex is ReflectionTypeLoadException)
                {
                    Console.WriteLine("LoaderExceptions: {0}", ((ReflectionTypeLoadException)ex).LoaderExceptions);
                }
            }
        }
    }
}
